from __future__ import annotations

import configparser
import json

from typing import Any, Mapping

import yaml

from jinjanator_plugins import Format, Formats, plugin_formats_hook


def _parse_ini(
    data_string: str,
    options: list[str] | None = None,  # noqa: ARG001
) -> Mapping[str, Any]:
    """INI data input format.

    data.ini:

    ```
    [nginx]
    hostname=localhost
    webroot=/var/www/project
    logs=/var/log/nginx
    ```

    Usage:

        $ j2 config.j2 data.ini
        $ cat data.ini | j2 --format=ini config.j2
    """

    # Override
    class MyConfigParser(configparser.ConfigParser):
        def as_dict(self) -> Mapping[str, Any]:
            d = dict(self._sections)  # type: ignore[attr-defined]
            for k in d:
                d[k] = dict(self._defaults, **d[k])  # type: ignore[attr-defined]
                d[k].pop("__name__", None)
            return d

    # Parse
    ini = MyConfigParser()
    ini.read_string(data_string)

    # Export
    return ini.as_dict()


def _parse_json(
    data_string: str,
    options: list[str] | None = None,  # noqa: ARG001
) -> Mapping[str, Any]:
    """JSON data input format.

    data.json:

    ```
    {
        "nginx":{
            "hostname": "localhost",
            "webroot": "/var/www/project",
            "logs": "/var/log/nginx"
        }
    }
    ```

    Usage:

        $ j2 config.j2 data.json
        $ cat data.json | j2 --format=ini config.j2
    """
    context = json.loads(data_string)

    if not isinstance(context, dict):
        msg = "JSON input does not contain an object (dictionary)"
        raise TypeError(msg)

    return context


def _parse_yaml(
    data_string: str,
    options: list[str] | None = None,  # noqa: ARG001
) -> Mapping[str, Any]:
    """YAML data input format.

    data.yaml:

    ```
    nginx:
      hostname: localhost
      webroot: /var/www/project
      logs: /var/log/nginx
    ```

    Usage:

        $ j2 config.j2 data.yml
        $ cat data.yml | j2 --format=yaml config.j2
    """
    context = yaml.safe_load(data_string)

    if not isinstance(context, dict):
        msg = "YAML input does not contain a mapping (dictionary)"
        raise TypeError(msg)

    return context


def _parse_env(
    data_string: str,
    options: list[str] | None = None,  # noqa: ARG001
) -> Mapping[str, str]:
    """Data input from environment variables.

    Render directly from the current environment variable values:

        $ j2 config.j2

    Or alternatively, read the values from a dotenv file:

    ```
    NGINX_HOSTNAME=localhost
    NGINX_WEBROOT=/var/www/project
    NGINX_LOGS=/var/log/nginx/
    ```

    And render with:

        $ j2 config.j2 data.env
        $ env | j2 --format=env config.j2

    If you're going to pipe a dotenv file into `j2`, you'll need to
    use "-" as the second argument to explicitly:

        $ j2 config.j2 - < data.env

    """
    # Parse
    return dict(
        filter(
            lambda line: len(line) == 2,  # noqa: PLR2004
            (
                list(map(str.strip, line.split("=", 1)))
                for line in data_string.split("\n")
            ),
        ),
    )


@plugin_formats_hook
def plugin_formats() -> Formats:
    return {
        "ini": Format(parser=_parse_ini, suffixes=[".ini"]),
        "json": Format(parser=_parse_json, suffixes=[".json"]),
        "yaml": Format(parser=_parse_yaml, suffixes=[".yaml", ".yml"]),
        "env": Format(parser=_parse_env, suffixes=[".env"]),
    }



from __future__ import annotations

from typing import Any

import pytest

import jinjanator.cli

from . import (
    FilePairFactory,
    render_file,
)


def test_quiet(make_file_pair: FilePairFactory, capsys: Any) -> None:
    files = make_file_pair("Hello {{name}}!", "name=Blart", "env")
    render_file(files, ["--quiet"])
    captured = capsys.readouterr()
    assert 0 == len(captured.err)


def test_unavailable_suffix(make_file_pair: FilePairFactory) -> None:
    files = make_file_pair("Hello {{name}}!", "name=Blart", "xyz")
    with pytest.raises(
        SystemExit,
        match="no format which can read '.xyz' files available",
    ):
        render_file(files, [])


def test_main_normal(make_file_pair: FilePairFactory, capsys: Any) -> None:
    files = make_file_pair("Hello {{name}}!", "name=Blart", "env")
    assert (
        jinjanator.cli.main(["--quiet", str(files.template_file), str(files.data_file)])
        is None
    )
    captured = capsys.readouterr()
    assert "Hello Blart!" == captured.out


def test_main_failure(make_file_pair: FilePairFactory) -> None:
    files = make_file_pair("Hello {{name}}!", "name=Blart", "xyz")
    assert 1 == jinjanator.cli.main(
        ["--quiet", str(files.template_file), str(files.data_file)]
    )



from __future__ import annotations

from typing import Any, Mapping

import pytest

from jinjanator_plugins import Format

from jinjanator.cli import parse_args


def fake_env_parser(
    data: str, options: list[str] | None = None  # noqa: ARG001
) -> Mapping[str, Any]:
    return {"foo": "bar"}


fake_env_format = Format(parser=fake_env_parser, suffixes=["env"])


def test_invalid_arg() -> None:
    """Ensure that an invalid argument is not accepted."""
    with pytest.raises(SystemExit):
        parse_args({}, ["--test-invalid-arg"])


@pytest.mark.parametrize(
    "args",
    [
        ["--format", "env"],
        ["--format-option", "opt"],
        ["--import-env", "env"],
        ["--output-file", "output"],
        ["--quiet"],
        ["--undefined"],
        ["-e", "env"],
        ["-f", "env"],
        ["-o", "output"],
    ],
)
def test_args(args: list[str]) -> None:
    """Ensure that known arguments are accepted."""
    parse_args({"env": fake_env_format}, [*args, "template"])


def test_version() -> None:
    """Ensure that '--version' argument is accepted and program exits without an error."""
    with pytest.raises(SystemExit) as excinfo:
        parse_args({}, ["--version"])
    assert 0 == excinfo.value.code


@pytest.mark.xfail()
@pytest.mark.parametrize(
    "args",
    [
        ["--format", "env", "-f", "env"],
        ["--import-env", "env", "-e", "env"],
    ],
)
def test_duplicate_args(args: list[str]) -> None:
    """Ensure that duplicate arguments are not accepted."""
    parse_args({"env": fake_env_format}, [*args, "template"])
